@@ -27,10 +27,10 @@ from copy import deepcopy
 import argcomplete
 from colorama import Fore, Style, init
 from ssh_para.version import __version__
-from .functions import addstr, addstrc, last_line, curses_init_pairs, CURSES_COLORS
-from .symbols import SYMBOL_BEGIN, SYMBOL_END, SYMBOL_PROG, SYMBOL_RES
-from .tui import launch_tui
-from .segment import Segment
+from ssh_para.functions import addstr, addstrc, last_line, curses_init_pairs, CURSES_COLORS
+from ssh_para.symbols import SYMBOL_BEGIN, SYMBOL_END, SYMBOL_PROG, SYMBOL_RES
+from ssh_para.tui import launch_tui
+from ssh_para.segment import Segment
 
 os.environ["TERM"] = "xterm-256color"
 
@@ -92,7 +92,6 @@ def parse_args() -> Namespace:
         "-d",
         "--dirlog",
         help="directory for ouput log files (default: ~/.ssh-para)",
-        default=os.path.expanduser("~/.ssh-para"),
     )
     parser.add_argument(
         "-m",
@@ -138,8 +137,9 @@ def parse_args() -> Namespace:
     host_group.add_argument(
         "-L",
         "--logs",
-        nargs="+",
+        nargs="*",
         help="""get latest/current ssh-para run logs
+-L [<runid>]               : launch log TUI of <runid> or latest run
 -L[<runid>/]*.out          : all hosts outputs
 -L[<runid>/]<host>.out     : command output of host
 -L[<runid>/]*.<status>     : command output of hosts <status>
@@ -150,7 +150,6 @@ default <runid> is latest ssh-para run (use -j <job> -d <dir> to access logs if
 <status>: [success,failed,timeout,killed,aborted]
 """,
     ).completer = log_choices  # type: ignore
-    parser.add_argument("-T", "--tui", action="store_true", help="log view in TUI")
     parser.add_argument("-s", "--script", help="script to execute")
     parser.add_argument("-a", "--args", nargs="+", help="script arguments")
 
@@ -280,7 +279,7 @@ class JobStatusLog:
 
     def open(self, dirlog: str, status: str) -> TextIOWrapper:
         """open log file for status"""
-        return open(f"{dirlog}/{status.lower()}.status", "w", encoding="UTF-8")
+        return open(f"{dirlog}/{status.lower()}.status", "w", encoding="UTF-8", buffering=1)
 
     def addhost(self, host: str, status: str) -> None:
         """add host in status log"""
@@ -316,6 +315,7 @@ class JobPrint(threading.Thread):
         timeout: float = 0,
         verbose: bool = False,
         maxhostlen: int = 15,
+        tui_cmd: str = "",
     ):
         """init properties / thread"""
         super().__init__()
@@ -328,6 +328,7 @@ class JobPrint(threading.Thread):
         self.nbjobs = nbjobs
         self.dirlog = dirlog
         self.aborted = []
+        self.tui_cmd = tui_cmd
         self.startsec = time()
         self.stdscr: Optional[curses._CursesWindow] = None
         self.paused = False
@@ -351,29 +352,8 @@ class JobPrint(threading.Thread):
         curses.noecho()
         curses.curs_set(0)
         curses.start_color()
+        curses_init_pairs()
         self.segment = Segment(self.stdscr, 5)
-        curses.init_pair(
-            self.status_color["RUNNING"], curses.COLOR_WHITE, curses.COLOR_BLUE
-        )
-        curses.init_pair(
-            self.status_color["RUNNING"] + 1, curses.COLOR_BLUE, curses.COLOR_BLACK
-        )
-        curses.init_pair(
-            self.status_color["SUCCESS"], curses.COLOR_WHITE, curses.COLOR_GREEN
-        )
-        curses.init_pair(
-            self.status_color["SUCCESS"] + 1, curses.COLOR_GREEN, curses.COLOR_BLACK
-        )
-        curses.init_pair(
-            self.status_color["FAILED"], curses.COLOR_WHITE, curses.COLOR_RED
-        )
-        curses.init_pair(
-            self.status_color["FAILED"] + 1, curses.COLOR_RED, curses.COLOR_BLACK
-        )
-        curses.init_pair(self.status_color["IDLE"], curses.COLOR_WHITE, 8)
-        curses.init_pair(self.status_color["IDLE"] + 1, 8, curses.COLOR_BLACK)
-        curses.init_pair(self.COLOR_GAUGE, 8, curses.COLOR_BLUE)
-        curses.init_pair(self.COLOR_HOST, curses.COLOR_YELLOW, curses.COLOR_BLACK)
 
     def killall(self) -> None:
         """kill all running threads pid"""
@@ -538,7 +518,15 @@ class JobPrint(threading.Thread):
                 f"ETA: {estimated}",
             ],
         )
-        addstr(self.stdscr, 1, 0, f" Dirlog: {self.pdirlog} Command: ")
+        printfile(
+            f"begin: {datetime.fromtimestamp(self.startsec).strftime('%Y-%m-%d %H:%M:%S')}",
+            f"end: --:--:--",
+            f"dur: {total_dur}",
+            f"runs: {nbend}/{self.nbjobs}",
+            f"\n{self.jobstatuslog.result()}",
+            file=f"{self.dirlog}/ssh-para.result",
+        )
+        addstr(self.stdscr, 1, 0, f" LogTUI: {self.tui_cmd} Command: ")
         addstrc(self.stdscr, self.cmd[: curses.COLS - self.stdscr.getyx()[1]])
         addstrc(self.stdscr, 2, 0, "")
         self.print_finished(line_num + (nbrun > 0))
@@ -673,6 +661,7 @@ class JobPrint(threading.Thread):
             print_tee(" ", jstatus.log, file=global_log)
         print_tee("command:", self.command, file=global_log)
         print_tee("log directory:", self.pdirlog, file=global_log)
+        print_tee("log TUI:", self.tui_cmd, "\n", file=global_log)
         start = datetime.fromtimestamp(self.startsec).strftime("%Y-%m-%d %H:%M:%S")
         print_tee(
             f"{nbrun}/{self.nbjobs} jobs run : begin: {start}",
@@ -819,7 +808,9 @@ def get_hosts(hostsfile: str, hosts: list) -> list:
         print("ERROR: ssh-para: No hosts definition", file=sys.stderr)
         sys.exit(1)
     if hostsfile == "-":
-        return list(filter(len, sys.stdin.read().splitlines()))
+        hosts = list(filter(len, sys.stdin.read().splitlines()))
+        os.dup2(os.open("/dev/tty", os.O_RDWR), 0)  # restore stdin
+        return hosts
     try:
         with open(hostsfile, "r", encoding="UTF-8") as fhosts:
             hosts = list(filter(len, fhosts.read().splitlines()))
@@ -958,7 +949,8 @@ def make_logdir(dirlog: str, job: str) -> str:
     jobdirlog = dirlog
     if job:
         jobdirlog += f"/{job}"
-    dirlogtime = jobdirlog + "/" + str(int(time()))
+    logtime = str(int(time()))
+    dirlogtime = os.path.join(jobdirlog, logtime)
     try:
         if not os.path.isdir(dirlogtime):
             os.makedirs(dirlogtime)
@@ -968,7 +960,7 @@ def make_logdir(dirlog: str, job: str) -> str:
     make_latest(dirlog, dirlogtime)
     if job:
         make_latest(jobdirlog, dirlogtime)
-    return dirlogtime
+    return (logtime, dirlogtime)
 
 
 def main() -> None:
@@ -985,18 +977,29 @@ def main() -> None:
         MAX_DOTS = args.maxdots
     if MAX_DOTS == -1:
         MAX_DOTS = None
+    dirlog = args.dirlog or os.path.expanduser("~/.ssh-para")
     if args.list:
-        log_results(args.dirlog, args.job)
-    if args.tui:
-        dirlog = os.path.join(args.dirlog, args.job)
+        log_results(dirlog, args.job)
+    tui = False
+    if args.logs:
+        try:
+            runid = int(args.logs[0])
+            tui = True
+        except ValueError:
+            pass
+    if tui or args.logs == []:
+        dirlog = os.path.join(dirlog, args.job)
         if args.logs:
-            dirlog = os.path.join(args.dirlog, args.logs[0])
+            dirlog = os.path.join(dirlog, args.logs[0])
         else:
             dirlog = get_latest_dir(dirlog)
+        if not isdir(dirlog):
+            print(f"Error: ssh-para: cannot access directory {dirlog}", file=sys.stderr)
+            sys.exit(1)
         launch_tui(dirlog)
         return
     if args.logs:
-        log_contents(args.logs, args.dirlog, args.job)
+        log_contents(args.logs, dirlog, args.job)
     if args.script:
         args.ssh_args.append(script_command(args.script, args.args))
         command = [args.script]
@@ -1012,7 +1015,13 @@ def main() -> None:
     else:
         hostsfile = "parameter"
     hosts = get_hosts(args.hostsfile, args.hosts)
-    dirlog = make_logdir(args.dirlog, args.job)
+    (runid, dirlog) = make_logdir(dirlog, args.job)
+    tui_command = " ".join([i for i in [
+        f"ssh-para",
+        f"-j {args.job}" if args.job else "",
+        f"-d {args.dirlog}" if args.dirlog else "",
+        f"-L {runid}",
+    ] if i])
     printfile(
         f"Hostsfile: {hostsfile} Command: {' '.join(command)}",
         file=f"{dirlog}/ssh-para.command",
@@ -1031,7 +1040,7 @@ def main() -> None:
     except AttributeError:
         pass
     p = JobPrint(
-        command, parallel, len(hosts), dirlog, args.timeout, args.verbose, max_len
+        command, parallel, len(hosts), dirlog, args.timeout, args.verbose, max_len, tui_command
     )
     p.start()
     jobruns = []
