@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-"""Simple curses TUI to browse ssh-para log directory.
+"""Simple curses TUI to browse run-para log directory.
 
 Features:
 - list jobs from a run directory (scan *.out files)
@@ -14,30 +14,45 @@ import os
 import re
 from glob import glob
 from typing import List, Dict, Optional
-from .functions import addstr, curses_init_pairs, CURSES_COLORS
+from .functions import addstr, curses_init_pairs, last_line, CURSES_COLORS
 from .segment import Segment
 from .symbols import SYMBOL_BEGIN, SYMBOL_END
 
 STATUSES = ["ALL", "SUCCESS", "FAILED", "TIMEOUT", "KILLED", "ABORTED"]
 
-def _read_tail(path: str, maxbytes: int = 4096) -> str:
+def _read_tail(path: str, maxbytes: Optional[int] = None) -> str:
     try:
         with open(path, "rb") as fd:
-            fd.seek(0, os.SEEK_END)
-            size = fd.tell()
-            start = max(0, size - maxbytes)
-            fd.seek(start)
+            if maxbytes:
+                fd.seek(0, os.SEEK_END)
+                size = fd.tell()
+                start = max(0, size - maxbytes)
+                fd.seek(start)
             data = fd.read()
         try:
-            return data.decode(errors="replace").strip()
+            return data.decode(errors="ignore").strip()
         except Exception:
             return ""
     except OSError:
         return ""
 
+def read_status_names(dirlog: str, file: str) -> List:
+    fpath = os.path.join(dirlog, file)
+    if os.path.exists(fpath):
+        with open(fpath, 'r') as f:
+            return f.read().splitlines()
+    return []
 
 def load_jobs(dirlog: str) -> List[Dict]:
     """Scan dirlog for job output files and build job entries."""
+    names_status = {}
+    for name in read_status_names(dirlog, "killed.status"):
+        names_status[name] = 'KILLED'
+    for name in read_status_names(dirlog, "timeout.status"):
+        names_status[name] = 'TIMEOUT'
+    for name in read_status_names(dirlog, "success.status"):
+        names_status[name] = 'SUCCESS'
+
     files = glob(os.path.join(dirlog, "*.out"))
     jobs: List[Dict] = []
     for f in sorted(files):
@@ -47,40 +62,40 @@ def load_jobs(dirlog: str) -> List[Dict]:
             continue
         status = "RUNNING"
         exit_code = ""
-        for s in STATUSES[1:]:
-            if os.path.exists(os.path.join(dirlog, f"{name}.{s.lower()}")):
-                status = s
-                if status == "FAILED":
-                    with open(os.path.join(dirlog, f"{name}.{s.lower()}"), "r", encoding="utf-8", errors="replace") as fd:
-                        exit_code = fd.read().strip().split()[2]
-                elif status == "SUCCESS":
-                    exit_code = "0"
-                elif status == "RUNNING":
-                    exit_code = ""
-                else:
-                    exit_code = "-1"
-                break
-        snippet = ""
-        tail = _read_tail(f, maxbytes=2048)
-        if tail:
-            lines = [l for l in tail.splitlines() if l.strip()]
-            snippet = lines[-1] if lines else ""
+        if name in names_status:
+            status = names_status[name]
+        else:
+            failed = os.path.join(dirlog, f"{name}.failed")
+            if os.path.exists(failed):
+                status = "FAILED"
+                with open(failed, "r", encoding="utf-8", errors="replace") as fd:
+                    exit_code = fd.read().strip().split()[2]
+        if status in ["ABORTED", "TIMEOUT", "KILLED"]:
+            exit_code = "255"
+        elif status == "SUCCESS":
+            exit_code = "0"
+        with open(f, "rb") as fd:
+            snippet = last_line(fd)
         jobs.append({
             "name": name,
-            "out": f,
             "status": status,
             "exit_code": exit_code,
             # "cmd": cmd or "",
             "snippet": snippet,
         })
+    aborted = os.path.join(dirlog, "aborted.status")
+    if os.path.exists(aborted):
+        with open(aborted, 'r') as f:
+            for name in f.read().splitlines():
+                jobs.append({"name": name, "status": "ABORTED", "exit_code": "-1", "snippet": "canceled run"})
     return jobs
 
 
 def parse_result(dirlog: str) -> Dict[str, str]:
     """Parse ssh-para.result file and return key summary values."""
     result_file = os.path.join(dirlog, "ssh-para.result")
-    summary: Dict[str, str] = { "begin": result_file, "end": str(os.path.isfile(result_file))}
-    # Only read the result file directly inside dirlog
+    summary: Dict[str, str] = {"end": "--:--:--"}
+
     if not os.path.isfile(result_file):
         return summary
     try:
@@ -89,23 +104,23 @@ def parse_result(dirlog: str) -> Dict[str, str]:
     except OSError:
         return summary
     # extract basic fields
-    import re as _re
 
-    m = _re.search(r"begin:\s*([0-9\- :]+)", text)
+    m = re.search(r"begin:\s*([0-9\- :]+)", text)
     if m:
         summary["begin"] = m.group(1).strip()
-    m = _re.search(r"end:\s*([0-9\- :]+)", text)
+    m = re.search(r"end:\s*([0-9\- :]+)", text)
     if m:
         summary["end"] = m.group(1).strip()
-    m = _re.search(r"dur:\s*([0-9:\.]+)", text)
+    m = re.search(r"dur:\s*([0-9:\.]+)", text)
     if m:
         summary["dur"] = m.group(1).strip()
-    m = _re.search(r"runs:\s*([0-9]+\s*/\s*[0-9]+)", text, _re.IGNORECASE)
+    m = re.search(r"runs:\s*([0-9]+\s*/\s*[0-9]+)", text, re.IGNORECASE)
     if m:
         summary["runs"] = m.group(1).strip()
+        summary["runs_total"] = summary["runs"].split("/")[1].strip()
     # counts
     for k in ("success", "failed", "timeout", "killed", "aborted"):
-        m = _re.search(rf"{k}:\s*([0-9]+)", text)
+        m = re.search(rf"{k}:\s*([0-9]+)", text)
         if m:
             summary[k] = m.group(1)
     return summary
@@ -120,62 +135,76 @@ class Tui:
         self.dirlog = dirlog
         self.jobs = load_jobs(dirlog)
         self.summary = parse_result(dirlog)
+        self.counts = self.summary.copy()
         self.name_filter = ""
         self.text_filter = ""
         self.name_re = None
         self.text_re = None
-        self.name_re_err = False
-        self.text_re_err = False
+        self.name_neg = False
+        self.text_neg = False
         self.status_idx = 0
         self.cursor = 0
         self.top = 0
+        self.filtered_jobs: Optional[List[Dict]] = None
+        self.statuses = ["ALL"]
         self.init_color()
-        with open(os.path.join(dirlog, "ssh-para.command"), "r", encoding="utf-8", errors="replace") as fd:
-           self.command = fd.read().strip().split("Command: ")[-1]
+        try:
+            with open(os.path.join(dirlog, "ssh-para.command"), "r", encoding="utf-8", errors="replace") as fd:
+                self.command = fd.read().strip().split("Command: ")[-1]
+        except Exception:
+            self.command = ""
 
+    def count(self, job: Dict) -> None:
+        if job["status"] != "ABORTED":
+            self.counts["runs"] += 1
+        if job["status"] == "SUCCESS":
+            self.counts["success"] += 1
+        elif job["status"] not in ["RUNNING", "ABORTED"]:
+            self.counts["failed"] += 1
 
     def filtered(self) -> List[Dict]:
-        s = STATUSES[self.status_idx]
+        if self.filtered_jobs is not None:
+            return self.filtered_jobs
+        self.counts = {"runs": 0, "success": 0, "failed": 0}
+        s = self.statuses[self.status_idx]
         res = []
         for j in self.jobs:
+            if j["status"] not in self.statuses:
+                self.statuses.append(j['status'])
             if s != "ALL" and j["status"] != s:
                 continue
             # Command filter: regex (compiled), fall back to substring if empty
             if self.name_filter:
-                if self.name_re_err:
-                    # invalid regex, treat as no match
+                if bool(self.name_re.search(j["name"] or "")) == self.name_neg:
                     continue
-                if self.name_re:
-                    if not self.name_re.search(j["name"] or ""):
-                        continue
             if self.text_filter:
-                if self.text_re_err:
-                    continue
-                # try snippet first, then tail
                 hay = j["snippet"] or ""
+                # filter: try snippet first, then tail
                 matched = False
                 matched_line = None
-                # prefer matching in the snippet line (fast)
-                if self.text_re and self.text_re.search(hay):
+                if self.text_re.search(hay):
                     matched = True
                     matched_line = hay
                 else:
-                    # search tail lines from end to start to find last matching line
-                    tail = _read_tail(j["out"], maxbytes=8192)
-                    if tail and self.text_re:
+                    outfile = os.path.join(self.dirlog, f"{j['name']}.out")
+                    tail = _read_tail(outfile)
+                    if tail:
                         for line in reversed(tail.splitlines()):
                             if self.text_re.search(line):
                                 matched = True
                                 matched_line = line
                                 break
-                if not matched:
+                if matched == self.text_neg:
                     continue
-                # show the last matching line as the snippet in results
                 j2 = j.copy()
-                j2["snippet"] = (matched_line or "").strip()
+                if not self.text_neg:
+                    j2["snippet"] = (matched_line or "").strip()
                 res.append(j2)
+                self.count(j2)
                 continue
             res.append(j)
+            self.count(j)
+        self.filtered_jobs = res
         return res
 
     def init_color(self) -> None:
@@ -184,17 +213,17 @@ class Tui:
         curses_init_pairs()
         self.segment = Segment(self.stdscr, 6)
 
-    def draw(self) -> None:
+    def draw(self, items: Optional[List[Dict]] = None) -> None:
         self.stdscr.erase()
         maxy, maxx = self.stdscr.getmaxyx()
         # summary line from ssh-para.result (display first if present)
         sumline = [
-            f"done: {self.summary['runs']}",
-            f"success: {self.summary['success']}",
-            f"failed: {self.summary['failed']}",
-            f"begin: {self.summary['begin']}",
-            f"end: {self.summary['end']}",
-            f"dur: {self.summary['dur']}"
+            f"runs: {self.counts["runs"]}/{self.summary.get('runs_total', '?')}",
+            f"success: {self.counts["success"]}",
+            f"failed: {self.counts["failed"]}",
+            f"begin: {self.summary.get('begin', '')}",
+            f"end: {self.summary.get('end', '')}",
+            f"dur: {self.summary.get('dur', '')}"
         ]
 
         # draw using Segment
@@ -203,9 +232,9 @@ class Tui:
         except Exception:
             self.stdscr.addnstr(0, 0, " | ".join(sumline), maxx - 1)
         first_item_line = 3
-        header = f"Filters: status={STATUSES[self.status_idx]} name='{self.name_filter}' text='{self.text_filter}' cmd={self.command}"
+        header = f"Filters: status={self.statuses[self.status_idx]} name='{self.name_filter}' text='{self.text_filter}' cmd={self.command}"
         self.stdscr.addnstr(1, 0, header, maxx - 1)
-        items = self.filtered()
+    # items already ensured above
         if not items:
             self.stdscr.addnstr(first_item_line, 0, "No matching jobs", maxx - 1)
             self.stdscr.refresh()
@@ -253,10 +282,11 @@ class Tui:
         # open a simple fullscreen viewer
         maxy, maxx = self.stdscr.getmaxyx()
         try:
-            with open(job["out"], "r", encoding="utf-8", errors="replace") as fd:
+            outfile = os.path.join(self.dirlog, f"{job['name']}.out")
+            with open(outfile, "r", encoding="utf-8", errors="replace") as fd:
                 lines = fd.read().splitlines()
         except OSError:
-            lines = ["(cannot open file)"]
+            lines = ["(no output file)"]
         pos = 0
         # search state (default to list-view text search if present)
         search_re = self.text_re if hasattr(self, "text_re") else None
@@ -340,10 +370,14 @@ class Tui:
             if ch in (ord('k'), curses.KEY_UP):
                 if pos > 0:
                     pos -= 1
-            if ch == curses.KEY_NPAGE:
+            if ch in (curses.KEY_NPAGE, 6):
                 pos = min(pos + h, max(0, len(lines) - h))
-            if ch == curses.KEY_PPAGE:
+            if ch in (curses.KEY_PPAGE, 2):
                 pos = max(0, pos - h)
+            if ch in (curses.KEY_HOME, ord('0')):
+                pos = 0 
+            if ch == curses.KEY_END or ch == ord('G'):
+                pos = max(0, len(lines) - h)
             if ch == ord('/'):
                 # prompt for a regexp and compile
                 expr = self.prompt("Search regexp: ")
@@ -404,75 +438,108 @@ class Tui:
             print(j.get("name", ""))
         print()
         # show active filters for context
-        print(f"Filters: status={STATUSES[self.status_idx]} name='{self.name_filter}' text='{self.text_filter}' cmd={getattr(self, 'command', '')}")
+        print(f"Filters: status={self.statuses[self.status_idx]} name='{self.name_filter}' text='{self.text_filter}' cmd={getattr(self, 'command', '')}")
         input("Press Enter to return to TUI...")
         # Reinitialize curses state
         self.init_curses()
-
+    
     def loop(self) -> None:
         curses.curs_set(0)
+        first_item_line = 3
         while True:
-            self.draw()
+            items = self.filtered()
+            items_len = len(items)
+            self.draw(items)
             ch = self.stdscr.getch()
             if ch in (ord('q'), 27):
                 break
             elif ch in (curses.KEY_DOWN, ord('j')):
-                if self.cursor + 1 < len(self.filtered()):
+                if self.cursor + 1 < items_len:
                     self.cursor += 1
             elif ch in (curses.KEY_UP, ord('k')):
                 if self.cursor > 0:
                     self.cursor -= 1
+            elif ch in (curses.KEY_NPAGE, 6):
+                # page down: move by visible page size
+                maxy, maxx = self.stdscr.getmaxyx()
+                step = max(1, maxy - 2 - first_item_line)
+                self.cursor = min(items_len - 1, self.cursor + step) if items_len else 0
+            elif ch in (curses.KEY_PPAGE, 2):
+                # page up: move by visible page size
+                maxy, maxx = self.stdscr.getmaxyx()
+                step = max(1, maxy - 2 - first_item_line)
+                self.cursor = max(0, self.cursor - step)
+            elif ch == curses.KEY_END or ch == ord('G'):
+                # jump to last job and make it visible
+                if items_len:
+                    self.cursor = items_len - 1
+                    maxy, maxx = self.stdscr.getmaxyx()
+                    avail = (maxy - 2) - first_item_line
+                    self.top = max(0, items_len - avail)
+            elif ch in (curses.KEY_HOME, ord('0')):
+                self.cursor = 0
             elif ch == ord('p'):
                 # show job names in console and wait for key
                 self.show_names_console()
             elif ch == ord('/'):
+                self.filtered_jobs = None
                 self.text_filter = self.prompt("Search text (regexp): ")
+                self.text_re = None
+                self.text_neg = False
                 # compile regex
-                if self.text_filter:
-                    try:
-                        self.text_re = re.compile(self.text_filter, re.IGNORECASE)
-                        self.text_re_err = False
-                    except re.error:
-                        self.text_re = None
-                        self.text_re_err = True
+                # support negation prefix: '!pattern' means exclude matches
+                if self.text_filter.startswith('!'):
+                    self.text_neg = True
+                    expr = self.text_filter[1:]
                 else:
-                    self.text_re = None
-                    self.text_re_err = False
+                    expr = self.text_filter
+                try:
+                    self.text_re = re.compile(expr, re.IGNORECASE)
+                except re.error:
+                    self.text_filter = ""
                 self.cursor = 0
                 self.top = 0
+                
             elif ch == ord('n'):
+                self.filtered_jobs = None
                 self.name_filter = self.prompt("Name filter (regexp): ")
+                self.name_neg = False
+                self.name_re = None
                 if self.name_filter:
-                    try:
-                        self.name_re = re.compile(self.name_filter, re.IGNORECASE)
-                        self.name_re_err = False
-                    except re.error:
-                        self.name_re = None
-                        self.name_re_err = True
-                else:
-                    self.name_re = None
-                    self.name_re_err = False
+                    # support negation prefix: '!pattern' means exclude matching names
+                    if self.name_filter.startswith('!'):
+                        self.name_neg = True
+                        expr = self.name_filter[1:]
+                    else:
+                        expr = self.name_filter
+                    if expr:
+                        try:
+                            self.name_re = re.compile(expr, re.IGNORECASE)
+                        except re.error:
+                            self.name_filter = ""
                 self.cursor = 0
                 self.top = 0
             elif ch == ord('s'):
-                self.status_idx = (self.status_idx + 1) % len(STATUSES)
+                self.filtered_jobs = None
+                self.status_idx = (self.status_idx + 1) % len(self.statuses)
                 self.cursor = 0
                 self.top = 0
             elif ch == ord('r'):
-                self.jobs = load_jobs(self.dirlog)
-                self.summary = parse_result(self.dirlog)
+                self.filtered_jobs = None
+                self.statuses = ["ALL"]
+                if self.summary["end"] == "--:--:--":
+                    self.jobs = load_jobs(self.dirlog)
+                    self.summary = parse_result(self.dirlog)
                 self.name_filter = ""
                 self.text_filter = ""
                 self.status_idx = 0
                 self.cursor = 0
                 self.top = 0
             elif ch in (curses.KEY_ENTER, 10, 13):
-                items = self.filtered()
                 if items:
                     job = items[self.cursor]
                     self.view_output(job)
 
-
 def launch_tui(dirlog: str) -> None:
     if not os.path.isdir(dirlog):
         raise FileNotFoundError(f"dirlog not found: {dirlog}")
